#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Mon May 15 19:19:11 2017

@author: lin
"""

from numpy import ceil,floor

def inter_ceil(t,j,v):
	return floor(v)+1

def inter_floor(t,j,v):
    return floor(v)